﻿Option Explicit On
Option Strict On
Imports PluginAPI
Imports System.Xml.Serialization
Imports System.IO
Imports System.Net.Sockets

Public Class Plugin
    Implements IPlugin
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '///                            SimTools Plugin - Edit the Setting below to provide support for you favorite game!                              ///
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '////////////////////////////////////////////////
    '/// Per Game Settings - Change for Each Game ///
    '////////////////////////////////////////////////
    Private Const _GameName As String = "IL2 Sturmovik" 'GameName - the displayed Name for this game. Also the DLL_Name ==> "Live for Speed.dll"
    Private Const _ProcessName As String = "il2fb" 'Process_Name without the (".exe") for this game
    public Const _Port As String = "19999"  'Your Sending/Recieving UDP Port for this game
    '////////////////////////////////////////////////
    '///           Memory Map Variables           ///
    '////////////////////////////////////////////////
    Private Const _Is_MemoryMap As Boolean = False 'Is a MemoryMap file Required for this game?
    Private Const _MMF_Name As String = "NULL" 'Set if using a Memory Map File - EXAMPLE("$gtr2$")
    '////////////////////////////////////////////////
    '///           MemoryHook Variables           ///
    '////////////////////////////////////////////////   
    Private Const _Is_MemoryHook As Boolean = False 'Is a Memory Hook Required for this game? 
    Private Const _MemHook_Roll As UInteger = 0 'Not Used = 0
    Private Const _MemHook_Pitch As UInteger = 0 'Not Used = 0
    Private Const _MemHook_Heave As UInteger = 0 'Not Used = 0
    Private Const _MemHook_Yaw As UInteger = 0 'Not Used = 0
    Private Const _MemHook_Sway As UInteger = 0 'Not Used = 0
    Private Const _MemHook_Surge As UInteger = 0 'Not Used = 0
    '////////////////////////////////////////////////
    '///    DOFs Used for Output for this Game    ///
    '////////////////////////////////////////////////
    Private Const _DOF_Support_Roll As Boolean = True
    Private Const _DOF_Support_Pitch As Boolean = True
    Private Const _DOF_Support_Heave As Boolean = False
    Private Const _DOF_Support_Yaw As Boolean = False
    Private Const _DOF_Support_Sway As Boolean = False
    Private Const _DOF_Support_Surge As Boolean = False

    'Used by GameManager when the Game Starts.
    Private Client As UdpClient
    Private GetData_sendBytes As [Byte]() = System.Text.Encoding.Default.GetBytes("R/30/36/38/42/44/46/48")
    Private tmr As New System.Timers.Timer()

    Private SendCnt As Integer = 0
    Private ReceiveCnt As Integer = 0

    Public Sub StartupCommands() Implements IPlugin.StartupCommands
        'DO SOMETHING HERE AT GAME START!
        'set client
        Client = New System.Net.Sockets.UdpClient(CInt(_Port))
        'timer setting
        tmr.Interval = 50
        tmr.Enabled = True
        tmr.Start()
        AddHandler tmr.Elapsed, AddressOf OnTimedEvent 'would this not go before start?
        MsgBox("Start. SendCount = " & SendCnt & "  ReceiveCount = " & ReceiveCnt)
    End Sub

    'Used by GameMnnager when the Game Stops.
    Public Sub ShutDownCommands() Implements IPlugin.ShutDownCommands
        'DO SOMETHING HERE AT GAME STOP!
        tmr.Enabled = False
        tmr.Stop()
        Client.Close()
        MsgBox("Stop. SendCount = " & SendCnt & "  ReceiveCount = " & ReceiveCnt)
    End Sub

    'Used by GameManager to Process a MemoryHook.
    Public Sub Process_MemoryHook() Implements IPlugin.Process_MemoryHook
        'DO SOMETHING HERE AT GAME START!
    End Sub

    'Used by GameManager to Process a MemoryMap.
    Public Sub Process_MemoryMap() Implements IPlugin.Process_MemoryMap
        'DO SOMETHING HERE AT GAME START!
    End Sub


    Private Sub OnTimedEvent(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs)
        Client.Send(GetData_sendBytes, GetData_sendBytes.Length, "localhost", 21000)

        SendCnt = SendCnt + 1

    End Sub

    'Used by GameEngine to Process Incoming UDP Packets.
    Private Speed_Input As Double = 0
    Private Turn_Input As Double = 0
    Private AngSpeed_Input As Double = 0
    Private Azimuth_Input As Double = 0
    Private BeacAzim_Input As Double = 0
    Private Roll_Input As Double = 0
    Private Pitch_Input As Double = 0

    Sub Process_PacketRecieved(Text As String) Implements IPlugin.Process_PacketRecieved
        Dim MyText As String
        Dim j, i As Integer

        ReceiveCnt = ReceiveCnt + 1

        MyText = ""
        For i = 0 To Text.Length - 1
            If Text(i) > " " Then
                If Text(i) <= "}" Then
                    MyText = MyText + Text(i)
                End If
            End If
        Next

        ' extract data from received string (not optimized yet)
        i = InStr(MyText, "/30\")
        j = InStr(MyText, "/36\")
        Speed_Input = CDbl(Mid(MyText, i + 4, j - i - 4))

        i = j
        j = InStr(MyText, "/38\")
        Turn_Input = CDbl(Mid(MyText, i + 4, j - i - 4))

        i = j
        j = InStr(MyText, "/42\")
        AngSpeed_Input = CDbl(Mid(MyText, i + 4, j - i - 4))

        i = j
        j = InStr(MyText, "/44\")
        Azimuth_Input = CDbl(Mid(MyText, i + 4, j - i - 4))

        i = j
        j = InStr(MyText, "/46\")
        BeacAzim_Input = CDbl(Mid(MyText, i + 4, j - i - 4))

        i = j
        j = InStr(MyText, "/48\")
        Roll_Input = CDbl(Mid(MyText, i + 4, j - i - 4))

        i = j
        j = MyText.Length
        Pitch_Input = CDbl(Mid(MyText, i + 4, j - i - 3))

        If Roll_Input > 90 Then
            Roll_Input = (180 - Roll_Input)
        End If
        If Roll_Input < -90 Then
            Roll_Input = (-180 - Roll_Input)
        End If


        If Pitch_Input > 90 Then
            Pitch_Input = (180 - Pitch_Input)
        End If
        If Pitch_Input < -90 Then
            Pitch_Input = (-180 - Pitch_Input)
        End If

        'assign the output to SimTools output vars
        Roll_Output = Roll_Input
        Pitch_Output = Pitch_Input
        Heave_Output = 0
        Yaw_Output = 0
        Surge_Output = 0
        Sway_Output = 0

        MsgBox("RECEIVE")

    End Sub

    'Used by GameManager to Patch a Game.
    Function PatchGame(ByVal MyPath As String, ByVal MyIp As String) As Boolean Implements IPlugin.PatchGame
        'Change as Needed
        'If game is already patched - Unpatch first
        UnPatch(MyPath)
        Try
            System.IO.File.Copy(MyPath & "conf.ini", MyPath & "cfg.BAK")
            SetAttr(MyPath & "conf.ini", GetAttr(MyPath & "conf.ini") And (Not vbReadOnly))

            'patch here - add 3 line to end of file
            Dim FILE_NAME = MyPath & "conf.ini"
            Dim i As Integer
            Dim aryText(2) As String
            aryText(0) = "[DeviceLink]"
            aryText(1) = "port=21000"
            aryText(2) = "host= " & MyIp
            Dim objWriter As New System.IO.StreamWriter(FILE_NAME, True)
            For i = 0 To 2
                objWriter.WriteLine(aryText(i))
            Next
            objWriter.Close()

            MsgBox("Patch Installed!", MsgBoxStyle.OkOnly, "Patching info")
            Return True
        Catch ex As Exception
            MsgBox("You Must first Install and Run the Game Once Prior to Patching" + vbCrLf + "Game Not Patched!", MsgBoxStyle.OkOnly, "Patching info")
            Return False
        End Try
    End Function

    'Used by GameManager to UnPatch a Game.
    Sub UnPatchGame(MyPath As String) Implements IPlugin.UnPatchGame
        'Change as Needed
        UnPatch(MyPath)
        MsgBox("Patch Uninstalled!", MsgBoxStyle.OkOnly, "Patching info")
    End Sub

    Private Sub UnPatch(MyPath As String)
        If System.IO.File.Exists(MyPath & "cfg.BAK") Then 'Restore backup File
            If System.IO.File.Exists(MyPath & "conf.ini") Then
                System.IO.File.Delete(MyPath & "conf.ini")
            End If
            Rename(MyPath & "cfg.BAK", MyPath & "conf.ini")
        End If
    End Sub

    Sub PatchPathInfo() Implements IPlugin.PatchPathInfo
        MsgBox("Please Select the IL-2's Installation Directory.", MsgBoxStyle.OkOnly, "Patching info")
    End Sub

    'Used by GameManager to Validate a Path befors Patching.
    Function ValidatePatchPath(MyPath As String) As Boolean Implements IPlugin.ValidatePatchPath
        'insert a simple validation of the patching path - let the user know he got it right
        If System.IO.File.Exists(MyPath & "\il2fb.exe") = True Then
            Return True
        Else
            Return False
        End If
    End Function
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    '///                                                        DO NOT EDIT BELOW HERE!!!                                                           ///
    '//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#Region "///// BUILT IN METHODS - DO NOT CHANGE /////"
    Public Function GetDOFsUsed() As String Implements IPlugin.GetDOFsUsed
        'Return DOF's Used (Roll,Pitch,Heave,Yaw,Sway,Surge)
        Return (_DOF_Support_Roll.ToString & "," & _DOF_Support_Pitch.ToString & "," & _DOF_Support_Heave.ToString & "," & _DOF_Support_Yaw.ToString & "," & _DOF_Support_Sway.ToString & "," & _DOF_Support_Surge.ToString)
    End Function

    'File Editor - Replaces a line in a txt/cfg file
    Private Sub MyFileEditor(FileName As String, LinetoEdit As String, StingReplacment As String)
        Dim lines As New List(Of String)
        Dim x As Integer = 0
        Using sr As New System.IO.StreamReader(FileName)
            While Not sr.EndOfStream
                lines.Add(sr.ReadLine)
            End While
        End Using

        For Each line As String In lines
            If line.Contains(LinetoEdit) Then
                lines.RemoveAt(x)
                lines.Insert(x, StingReplacment)
                Exit For 'must exit as we changed the iteration   
            End If
            x = x + 1
        Next

        Using sw As New System.IO.StreamWriter(FileName)
            For Each line As String In lines
                sw.WriteLine(line)
            Next
        End Using
        lines.Clear()
    End Sub

    Public Sub ResetDOFVars() Implements IPlugin.ResetDOFVars
        Roll_Output = 0
        Pitch_Output = 0
        Heave_Output = 0
        Yaw_Output = 0
        Sway_Output = 0
        Surge_Output = 0
    End Sub

    Public Sub ResetMapVars() Implements IPlugin.ResetMapVars
        Roll_Output = 0
        Pitch_Output = 0
        Heave_Output = 0
        Yaw_Output = 0
        Sway_Output = 0
        Surge_Output = 0
    End Sub

    Public Sub ResetHookVars() Implements IPlugin.ResetHookVars
        Roll_Output = 0
        Pitch_Output = 0
        Heave_Output = 0
        Yaw_Output = 0
        Sway_Output = 0
        Surge_Output = 0
    End Sub

    Public ReadOnly Property GameName() As String Implements IPlugin.GameName
        Get
            Return _GameName
        End Get
    End Property


    Public ReadOnly Property Name() As String Implements IPlugin.Name
        Get
            Return _GameName
        End Get
    End Property

    Public ReadOnly Property ProcessName() As String Implements IPlugin.ProcessName
        Get
            Return _ProcessName
        End Get
    End Property

    Public ReadOnly Property Port() As String Implements IPlugin.Port
        Get
            Return _Port
        End Get
    End Property

    Public ReadOnly Property Is_MemoryMap() As Boolean Implements IPlugin.Is_MemoryMap
        Get
            Return _Is_MemoryMap
        End Get
    End Property

    Public ReadOnly Property Is_MemoryHook() As Boolean Implements IPlugin.Is_MemoryHook
        Get
            Return _Is_MemoryHook
        End Get
    End Property

    Public Function Get_RollOutput() As Double Implements IPlugin.Get_RollOutput
        Return Roll_Output
    End Function

    Public Function Get_PitchOutput() As Double Implements IPlugin.Get_PitchOutput
        Return Pitch_Output
    End Function

    Public Function Get_HeaveOutput() As Double Implements IPlugin.Get_HeaveOutput
        Return Heave_Output
    End Function

    Public Function Get_YawOutput() As Double Implements IPlugin.Get_YawOutput
        Return Yaw_Output
    End Function

    Public Function Get_SwayOutput() As Double Implements IPlugin.Get_SwayOutput
        Return Sway_Output
    End Function

    Public Function Get_SurgeOutput() As Double Implements IPlugin.Get_SurgeOutput
        Return Surge_Output
    End Function

    Public Function Get_RollMemHook() As Double Implements IPlugin.Get_RollMemHook
        Return Roll_MemHook
    End Function

    Public Function Get_PitchMemHook() As Double Implements IPlugin.Get_PitchMemHook
        Return Pitch_MemHook
    End Function

    Public Function Get_HeaveMemHook() As Double Implements IPlugin.Get_HeaveMemHook
        Return Heave_MemHook
    End Function

    Public Function Get_YawMemHook() As Double Implements IPlugin.Get_YawMemHook
        Return Yaw_MemHook
    End Function

    Public Function Get_SwayMemHook() As Double Implements IPlugin.Get_SwayMemHook
        Return Sway_MemHook
    End Function

    Public Function Get_SurgeMemHook() As Double Implements IPlugin.Get_SurgeMemHook
        Return Surge_MemHook
    End Function

    Public Function Get_RollMemMap() As Double Implements IPlugin.Get_RollMemMap
        Return Roll_MemMap
    End Function

    Public Function Get_PitchMemMap() As Double Implements IPlugin.Get_PitchMemMap
        Return Pitch_MemMap
    End Function

    Public Function Get_HeaveMemMap() As Double Implements IPlugin.Get_HeaveMemMap
        Return Heave_MemMap
    End Function

    Public Function Get_YawMemMap() As Double Implements IPlugin.Get_YawMemMap
        Return Yaw_MemMap
    End Function

    Public Function Get_SwayMemMap() As Double Implements IPlugin.Get_SwayMemMap
        Return Sway_MemMap
    End Function

    Public Function Get_SurgeMemMap() As Double Implements IPlugin.Get_SurgeMemMap
        Return Surge_MemMap
    End Function

    'For MemoryHook
    Public Declare Function ReadProcessMemory Lib "kernel32" Alias "ReadProcessMemory" (ByVal hProcess As Integer, ByVal lpBaseAddress As Integer, ByVal lpBuffer() As Byte, ByVal nSize As Integer, ByRef lpNumberOfBytesWritten As Integer) As Integer
    Public Declare Function CloseHandle Lib "kernel32" Alias "CloseHandle" (ByVal hObject As Integer) As Integer
    Public Declare Function OpenProcess Lib "kernel32" Alias "OpenProcess" (ByVal dwDesiredAccess As Integer, ByVal bInheritHandle As Integer, ByVal dwProcessId As Integer) As Integer

    'Output Vars
    Public Roll_Output As Double = 0
    Public Pitch_Output As Double = 0
    Public Heave_Output As Double = 0
    Public Yaw_Output As Double = 0
    Public Sway_Output As Double = 0
    Public Surge_Output As Double = 0

    'MemHook Vars
    Public Roll_MemHook As Double = 0
    Public Pitch_MemHook As Double = 0
    Public Heave_MemHook As Double = 0
    Public Yaw_MemHook As Double = 0
    Public Sway_MemHook As Double = 0
    Public Surge_MemHook As Double = 0

    'MemMap Vars
    Public Roll_MemMap As Double = 0
    Public Pitch_MemMap As Double = 0
    Public Heave_MemMap As Double = 0
    Public Yaw_MemMap As Double = 0
    Public Sway_MemMap As Double = 0
    Public Surge_MemMap As Double = 0

    'DeSerialize
    Public Function DeSerializeString(InputString As String) As Object
        On Error Resume Next
        Dim xml_serializer As New XmlSerializer(GetType(GenericOutput))
        Dim string_reader As New StringReader(InputString)
        Dim DeserializedOutput As GenericOutput = DirectCast(xml_serializer.Deserialize(string_reader), GenericOutput)
        string_reader.Close()
        Return DeserializedOutput
    End Function

    'Generic In/Output structure
    Public Structure GenericOutput
        Public Roll As Single
        Public Pitch As Single
        Public Heave As Single
        Public Yaw As Single
        Public Sway As Single
        Public Surge As Single
    End Structure

    'memory hook function #1
    Public Function ReadSingle(hProcess As String, dwAddress As UInteger) As String
        Try
            Dim proc As Process = Process.GetProcessesByName(hProcess)(0)
            Dim winhandle As Integer = CInt(CType(OpenProcess(&H1F0FFF, 1, proc.Id), IntPtr))
            Dim buffer As Byte() = New Byte(3) {}
            buffer.ToArray()
            ReadProcessMemory(winhandle, CInt(dwAddress), buffer, 4, 0)
            Dim MySingle As Single = BitConverter.ToSingle(buffer, 0)
            Return MySingle.ToString
        Catch ex As Exception
            Return Nothing
        End Try
    End Function

    'memory hook function #2
    Public Function ReadInt32(hProcess As String, dwAddress As UInteger) As String
        Try
            Dim proc As Process = Process.GetProcessesByName(hProcess)(0)
            Dim winhandle As Integer = CInt(CType(OpenProcess(&H1F0FFF, 1, proc.Id), IntPtr))
            Dim buffer As Byte() = New Byte(3) {}
            buffer.ToArray()
            ReadProcessMemory(winhandle, CInt(dwAddress), buffer, 4, 0)
            Dim MyInt32 As Integer = BitConverter.ToInt32(buffer, 0)
            Return MyInt32.ToString
        Catch ex As Exception
            Return Nothing
        End Try
    End Function
#End Region
End Class
